/*                                                                            */

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	#include <proto/exec.h>
	#include <proto/dos.h>
	#include <proto/graphics.h>
	#include <proto/intuition.h>
	#include <proto/datatypes.h>
	#include <datatypes/pictureclass.h>
	#include <clib/alib_protos.h>

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	struct BackfillMsg {
		struct Layer *layer;
		struct Rectangle bounds;
		LONG offsetx;
		LONG offsety;
		};

	struct bgimg {
		Object *o;
		struct BitMap *bm;
		UWORD w;
		UWORD h;
		};

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	struct bgimg *load_image (char *name,struct Screen *scr)

	{
	struct bgimg *img;

	if (img = AllocVec (sizeof(struct bgimg),MEMF_CLEAR))
		{
		Object *dto;

		if (dto = NewDTObject (name,
				DTA_GroupID,GID_PICTURE,
				PDTA_Screen,scr,
				PDTA_Remap,TRUE,
				PDTA_DestMode,PMODE_V43,
				TAG_END))
			{
			struct BitMap *bm = NULL;
			struct BitMapHeader *bmhd = NULL;

			DoMethod (dto,DTM_PROCLAYOUT,NULL,TRUE);

			GetDTAttrs (dto,
				PDTA_BitMap,&bm,
				PDTA_BitMapHeader,&bmhd,
				TAG_END);

			if (bm && bmhd)
				{
				img->bm = bm;
				img->w  = bmhd->bmh_Width;
				img->h  = bmhd->bmh_Height;
				img->o  = dto;

				return (img);
				}

			DisposeObject (dto);
			}

		FreeVec (img);
		}

	return (NULL);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	void free_image (struct bgimg *img)

	{
	if (img)
		{
		if (img->o)
			DisposeObject (img->o);

		FreeVec (img);
		}
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	long min (long a,long b)

	{
	return (a < b ? a : b);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

#ifndef __PPC__
	__saveds
#endif
	void backfill_funct (struct Hook *hook,struct RastPort *rp,struct BackfillMsg *msg)

	{
	struct bgimg *img = (struct bgimg *) hook->h_Data;
	long ox,oy;
	long w,h;
	long x,y;
	long x1,y1;

	if (!img)
		return;

	x1 = msg->bounds.MaxX + 1;
	y1 = msg->bounds.MaxY + 1;

	oy = msg->offsety % img->h;
	y = msg->bounds.MinY;
	while (y <= msg->bounds.MaxY)
		{
		h = min(y1 - y,img->h - oy);

		ox = msg->offsetx % img->w;
		x = msg->bounds.MinX;
		while (x <= msg->bounds.MaxX)
			{
			w = min(x1 - x,img->w - ox);
			BltBitMap (img->bm,ox,oy,rp->BitMap,x,y,w,h,ABC|ABNC,0xff,NULL);
			x += w;
			ox = 0;
			}

		y += h;
		oy = 0;
		}
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	int main (void)

	{
	struct RDArgs *rdargs;
	struct {
		char *file;
		char *pubscreen;
		} args = {"Workbench:Prefs/Presets/Patterns/Eyestrain/PaislyBlue.jpg",NULL};
	struct Screen *scr;

	rdargs = ReadArgs ("FILE,PUBSCREEN/K",(LONG *)&args,NULL);
	if (!rdargs)
		{
		PrintFault (IoErr(),NULL);
		return (RETURN_FAIL);
		}

	if (scr = LockPubScreen (args.pubscreen))
		{
		struct bgimg *img;

		if (img = load_image (args.file,scr))
			{
			struct Hook bfhook;
			struct Window *win;

		#ifdef __PPC__
			bfhook.h_Entry    = (HOOKFUNC)backfill_funct;
		#else
			bfhook.h_Entry    = (HOOKFUNC)HookEntry;
			bfhook.h_SubEntry = (HOOKFUNC)backfill_funct;
		#endif
			bfhook.h_Data     = (APTR)img;

			if (win = OpenWindowTags (NULL,
					WA_Left,scr->Width / 4,
					WA_Top,scr->Height / 4,
					WA_Width,scr->Width / 2,
					WA_Height,scr->Height / 2,
					WA_MinWidth,60,
					WA_MinHeight,40,
					WA_MaxWidth,0x7fff,
					WA_MaxHeight,0x7fff,
					WA_Flags,WFLG_CLOSEGADGET | WFLG_DRAGBAR | WFLG_DEPTHGADGET | WFLG_SIZEGADGET | WFLG_ACTIVATE | WFLG_NOCAREREFRESH,
					WA_IDCMP,IDCMP_CLOSEWINDOW | IDCMP_VANILLAKEY,
					WA_Title,"Backfill Example",
					WA_BackFill,&bfhook,
					TAG_END))
				{
				BOOL cont = TRUE;

				do	{
					struct IntuiMessage *imsg;

					if (Wait ((1L << win->UserPort->mp_SigBit) | SIGBREAKF_CTRL_C) & SIGBREAKF_CTRL_C)
						cont = FALSE;

					while (imsg = (struct IntuiMessage *) GetMsg (win->UserPort))
						{
						switch (imsg->Class)
							{
						case IDCMP_VANILLAKEY:
							if (imsg->Code == 0x1b) /* Esc */
								cont = FALSE;
							break;
						case IDCMP_CLOSEWINDOW:
							cont = FALSE;
							break;
							}
						ReplyMsg ((struct Message *) imsg);
						}
					}
				while (cont);

				CloseWindow (win);
				}

			free_image (img);
			}

		UnlockPubScreen (NULL,scr);
		}

	FreeArgs (rdargs);

	return (RETURN_OK);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/
